package org.expeditee.greenstone;


import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class hcibibGUI implements ItemListener, ActionListener {

    /*  You should not need to modify this set of global variables */
    JFrame mainFrame;
    Greenstone3Connection gsdl;
    JCheckBox stemmingCheck;
    JCheckBox casefoldingCheck;
    JTextField queryField;
    JTextField docsToReturnField;
    JComboBox indexComboBox;
    JRadioButton homeButton;
    JRadioButton waikatoButton;
    JEditorPane resultPane;
    boolean doCasefolding = true;
    boolean doStemming = false;
    boolean getAllMetadata = true;
    int indexChoice = 1;
    int locationChoice = 1;
    String[] indexKeys = {"TX", "TI", "JO", "BO", "CR", "KE"};
    Vector currentResultSet;
    /* If you need to, you can introduce further globals to store useful data  */
    /** 
     *  dateMap is a hash table. The keys are year values.
     *  the data associated with each key is a Vector of document IDs
     *  therefore, for the current result set you can get the set of years in which the
     *  results were published, and for each year you can get the set of documents published in that year
     *  
     *  If you want to introduce additional mappings (eg document written by authors) you should
     *  introduce additional structures here (HashMap used in the same way as dateMap will probably suffice
     *  
     */
    HashMap dateMap = new HashMap();

    /* creates the result window which contains a menu and HTML renderer
     * you will only need to modify this code to add more items to the View menu
     *  or add additional GUI components to the window (which isn't required for the assignment
     *  To change the initial window size modify the arguments to the setSize()  call at end of this method
     */
    public void createAndShowResultGUI() {
        JFrame resultFrame = new JFrame("Query results");
        JPanel containerPanel = new JPanel();
        JPanel controlPanel = new JPanel();
        GridBagConstraints gbc = new GridBagConstraints();

        containerPanel.setLayout(new GridBagLayout());
        resultFrame.setContentPane(containerPanel);

        containerPanel.setLayout(new GridBagLayout());
        controlPanel.setLayout(new GridBagLayout());

        JMenuBar menuBar = new JMenuBar();
        JMenu viewMenu = new JMenu("View");
        menuBar.add(viewMenu);




        /* add items to the view menu. Insert code here to add more items */
        JMenuItem viewByDate = new JMenuItem("View by Date");
        viewByDate.addActionListener(this);
        viewMenu.add(viewByDate);

        JMenuItem viewByScore = new JMenuItem("View by Score");
        viewByScore.addActionListener(this);
        viewMenu.add(viewByScore);
        /* end of menu item code */




        resultFrame.setJMenuBar(menuBar);

        resultPane = new JEditorPane();
        resultPane.setContentType("text/html");
        resultPane.setEditable(false);
        resultPane.addHyperlinkListener(new HyperlinkListener() {

            public void hyperlinkUpdate(HyperlinkEvent r) {
                try {
                    if (r.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                        handleLinkClick(r);
                    }
                } catch (Exception e) {
                }
            }
        });
        resultPane.setText("Results will appear here");

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 0.0;
        gbc.anchor = GridBagConstraints.LINE_START;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        containerPanel.add(controlPanel, gbc);

        gbc.gridy = 1;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        containerPanel.add(new JScrollPane(resultPane), gbc);

        resultFrame.pack();
        resultFrame.setSize(400, 600);
        resultFrame.setVisible(true);
    }

    /* the argument to this method is the label of an item selected in the View menu
     * based on the item selected from the menu the appropriate view method is called
     * to handle additional view options add further else-if clauses
     * you will need to implement any further viewByXXXX methods that you call
     */
    public void changeView(String menuLabel) {
        if (menuLabel.compareTo("View by Date") == 0) {
            viewByDate(dateMap);
        } else if (menuLabel.compareTo("View by Score") == 0) {
            viewByScore(currentResultSet);
        }
    }

    /* this method is called when any link in the result window is clicked
     * It expects the link URL to be of the form    http://<metadata-tag>#<document-id>
     * For example    http://title#HASH01f33a09ebd2421ae5155b37s55
     * indicates that the user has clicked on the title link for document with ID HASH01f33a09ebd2421ae5155b37s55
     * 
     * This enables you to make the application respond to clicks within result items if you wish to modify
     * the view based on such a user action
     *  
     * Note that you need to construct these links in code that generates the HTML to be rendered in the result window
     */
    public void handleLinkClick(HyperlinkEvent r) {
        String linkText = r.getURL().toString();
        linkText = linkText.replaceAll("http://", "");
        String parts[] = linkText.split("#");

        ResultDocument rd = gsdl.getDocument(parts[1]);

        StringBuffer HTMLcontent = new StringBuffer("<html><body>");
        HTMLcontent.append("You clicked on the " + parts[0] + " of document " + parts[1] + "<br>");
        HTMLcontent.append("Document title = " + rd.getTitle());
        HTMLcontent.append("</body></html>");

        resultPane.setText(HTMLcontent.toString());
    }

    /* given the Vector of result items (ordered by descending relevance to the query) this method
     * iterates through them and constructs an HTML document and has it rendered in the result window
     * 
     * This is the default presentation for results
    
     * You can modify this method as you wish to change the HTML for the default presentation
     * 
     * Some useful method calls are illustrated here
     *       - if you have the ID of a document you can get all data stored for it with
     *           ResultDocument rd = gsdl.getDocument(docID);
     *       - a ResultDocument object has a set of methods for getting metadata values for that document
     *           most metadata is a single value, but authors and keywords can have multiple values
     *           and these are stored in a Vector
     * 
     * The IF condition for 'Title' shows how to construct a link that can be clicked on
     * and then dealt with by the    handleLinkClick()   method
     * 
     */
    public void viewByScore(Vector results) {
        StringBuffer resultHTML = new StringBuffer("<html><head>Results</head><body>");

        for (Enumeration e = results.elements(); e.hasMoreElements();) {
            Result result = (Result) e.nextElement();

            String docID = result.getDocID();
            int docRank = result.getRank();
            String docScore = result.getScore();

            ResultDocument rd = gsdl.getDocument(docID);

            resultHTML.append("<p>");
            resultHTML.append("docID = " + docID + "<br>");
            resultHTML.append("docRank = " + docRank + " (for this query)<br>");
            resultHTML.append("docScore = " + docScore + " (for this query)<br>");

            if (rd.metadataExists("Title")) {
                resultHTML.append("title = <a  href='http://title#" + docID + "'>" + rd.getTitle() + "</a><br>");
            }
            if (rd.metadataExists("Date")) {
                resultHTML.append("date = " + rd.getDate() + "<br>");
            }
            if (rd.metadataExists("Booktitle")) {
                resultHTML.append("booktitle = " + rd.getBooktitle() + "<br>");
            }
            if (rd.metadataExists("Pages")) {
                resultHTML.append("pages = " + rd.getPages() + "<br>");
            }
            if (rd.metadataExists("Journal")) {
                resultHTML.append("journal = " + rd.getJournal() + "<br>");
            }
            if (rd.metadataExists("Volume")) {
                resultHTML.append("volume = " + rd.getVolume() + "<br>");
            }
            if (rd.metadataExists("Number")) {
                resultHTML.append("number = " + rd.getNumber() + "<br>");
            }
            if (rd.metadataExists("Editor")) {
                resultHTML.append("editor = " + rd.getEditor() + "<br>");
            }
            if (rd.metadataExists("Publisher")) {
                resultHTML.append("publisher = " + rd.getPublisher() + "<br>");
            }
            if (rd.metadataExists("Abstract")) {
                resultHTML.append("abstract = " + rd.getAbstract() + "<br>");
            }
            Vector authors = rd.getAuthors();
            for (Enumeration a = authors.elements(); a.hasMoreElements();) {
                String author = (String) a.nextElement();
                resultHTML.append("author = " + author + "<br>");
            }
            Vector keywords = rd.getKeywords();
            for (Enumeration k = keywords.elements(); k.hasMoreElements();) {
                String keyword = (String) k.nextElement();
                resultHTML.append("keyword = " + keyword + "<br>");
            }
            resultHTML.append("</p>");
        }
        resultHTML.append("</body></html>");

        resultPane.setText(resultHTML.toString());
        resultPane.setCaretPosition(0);
    }

    /* This method provides an alternative view of the result set
     * It uses the dateMap which organises results according to the year in which they were published
     * 
     * You will need to write a similar method for any additional view that you implement
     * 
     * You may wish to modify this method to provide a better layout of the output
     */
    public void viewByDate(HashMap dateMap) {
        StringBuffer resultHTML = new StringBuffer("<html><head>Results</head><body>");

        Vector theDates = new Vector(dateMap.keySet());
        Collections.sort(theDates);


        for (Enumeration e = theDates.elements(); e.hasMoreElements();) {
            String date = (String) e.nextElement();
            Vector ids = (Vector) dateMap.get(date);

            resultHTML.append("<p>");
            resultHTML.append("<b>" + date + "</b><br>");

            for (Enumeration i = ids.elements(); i.hasMoreElements();) {
                String docID = (String) i.nextElement();
                ResultDocument rd = gsdl.getDocument(docID);
                resultHTML.append(rd.getTitle() + "<br>");
            }

            resultHTML.append("</p>");
        }

        resultHTML.append("</body></html>");

        resultPane.setText(resultHTML.toString());
        resultPane.setCaretPosition(0);
    }

    /* this method is called whenever the user presses the RETURN key in the query box
     * 
     * it first deletes any date->document mappings from the previous query by clearing the dateMap hashtable
     * if you introduce further mapping structures you will need to clear them here
     * 
     * a query object is created using the settings in the query window and the query request is
     * sent to the Greenstone server, which responds with the outcome of the query
     * the metadata for each result document is then retrieved from the server
     * 
     * The dateMap hashtable is then initialised
     * 
     * If you wished to group documents by author you would
     *      - introduce a new HashMap (probably called   authorMap  )
     *      - write a method called   initialiseAuthorMap    modelled on initialiseDateMap
     *          which uses a method   addToAuthorMap  which would be modelled on   addToDateMap
     *      - call   initialiseDateMap  in this method
     *      - write a viewByAuthor method
     * 
     * the  viewByScore   method is called to display the default result output
     * you can change the view method which is called if you wish to
     */
    public void doQuery(String queryText) {

        dateMap.clear();

        Query query = createQuery(queryText);
        QueryOutcome queryOutcome = gsdl.issueQueryToServer(query);
        currentResultSet = getResultSetMetadata(queryOutcome);

        initialiseDateMap(currentResultSet);

        viewByScore(currentResultSet);
    }

    /************** YOU SHOULD NOT NEED TO MODIFY ANYTHING PAST THIS POINT  *************/
    public static void main(String[] args) {
        hcibibGUI gui = new hcibibGUI();
    }

    public hcibibGUI() {
        gsdl = new Greenstone3Connection(locationChoice);
        createAndShowMainGUI();
        createAndShowResultGUI();
    }

    public void createAndShowMainGUI() {
        JPanel containerPanel = new JPanel();
        JPanel locationPanel = new JPanel();
        JPanel optionPanel = new JPanel();
        JPanel queryPanel = new JPanel();
        GridBagConstraints gbc = new GridBagConstraints();


        mainFrame = new JFrame("GUI for Greenstone 3 HCI Bibliography Collection");
        containerPanel.setLayout(new GridBagLayout());
        mainFrame.setContentPane(containerPanel);

        locationPanel.setLayout(new GridBagLayout());
        optionPanel.setLayout(new GridBagLayout());
        queryPanel.setLayout(new GridBagLayout());

        TitledBorder title;
        title = BorderFactory.createTitledBorder("Where am I?");
        locationPanel.setBorder(title);
        title = BorderFactory.createTitledBorder("Query options");
        optionPanel.setBorder(title);
        title = BorderFactory.createTitledBorder("Query");
        queryPanel.setBorder(title);

        ButtonGroup locationGroup = new ButtonGroup();
        homeButton = new JRadioButton("home");
        waikatoButton = new JRadioButton("CS labs");
        homeButton.addActionListener(this);
        waikatoButton.addActionListener(this);
        homeButton.setSelected(true);
        locationGroup.add(homeButton);
        locationGroup.add(waikatoButton);

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.LINE_START;
        locationPanel.add(homeButton, gbc);
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.LINE_START;
        locationPanel.add(waikatoButton, gbc);

        stemmingCheck = new JCheckBox("Stemming");
        stemmingCheck.setSelected(doStemming);
        stemmingCheck.addItemListener(this);
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.LINE_START;
        optionPanel.add(stemmingCheck, gbc);

        casefoldingCheck = new JCheckBox("Casefolding");
        casefoldingCheck.setSelected(doCasefolding);
        casefoldingCheck.addItemListener(this);
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.LINE_START;
        optionPanel.add(casefoldingCheck, gbc);

        JLabel docsToReturnLabel = new JLabel("Max results");
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.LINE_END;
        optionPanel.add(docsToReturnLabel, gbc);

        docsToReturnField = new JTextField(3);
        docsToReturnField.setText("10");
        gbc.gridx = 2;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.LINE_START;
        optionPanel.add(docsToReturnField, gbc);

        JLabel indexLabel = new JLabel("Search in...");
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.anchor = GridBagConstraints.LINE_START;
        optionPanel.add(indexLabel, gbc);

        String[] indexStrings = {"all fields", "document titles", "journal names",
            "book/conference names", "authors", "keywords"
        };
        indexComboBox = new JComboBox(indexStrings);
        indexComboBox.setSelectedIndex(1);
        indexComboBox.addActionListener(this);
        gbc.gridx = 0;
        gbc.gridy = 3;
        optionPanel.add(indexComboBox, gbc);

        queryField = new JTextField(30);
        queryField.addActionListener(new MyActionListener());
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.LINE_START;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        queryPanel.add(queryField, gbc);

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 0.0;
        gbc.anchor = GridBagConstraints.LINE_START;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        containerPanel.add(locationPanel, gbc);
        gbc.gridy = 1;
        containerPanel.add(optionPanel, gbc);
        gbc.gridy = 2;
        containerPanel.add(queryPanel, gbc);

        mainFrame.pack();
        mainFrame.setVisible(true);
    }

    public Query createQuery(String queryText) {

        Query query = new Query();

        // set the query options
        query.setQueryText(queryText);

        query.setIndex(indexKeys[indexChoice]);

        String docsToReturn = docsToReturnField.getText();
        if (docsToReturn.compareTo("") == 0) {
            docsToReturn = "10";
        }
        query.setMaxDocsToReturn(docsToReturn);

        if (doStemming) {
            query.setStemming("1");
        } else {
            query.setStemming("0");
        }

        if (doCasefolding) {
            query.setCasefolding("1");
        } else {
            query.setCasefolding("0");
        }

        return query;
    }

    private void getResultMetadata(Result result) {
        String docID = result.getDocID();

        gsdl.getDocumentMetadataFromServer(docID, "Title");
        gsdl.getDocumentMetadataFromServer(docID, "Date");

        if (getAllMetadata) {
            gsdl.getDocumentMetadataFromServer(docID, "Date");
            gsdl.getDocumentMetadataFromServer(docID, "Booktitle");
            gsdl.getDocumentMetadataFromServer(docID, "Journal");
            gsdl.getDocumentMetadataFromServer(docID, "Creator");
            gsdl.getDocumentMetadataFromServer(docID, "Keywords");
            gsdl.getDocumentMetadataFromServer(docID, "Publisher");
            gsdl.getDocumentMetadataFromServer(docID, "Abstract");
            gsdl.getDocumentMetadataFromServer(docID, "Pages");
            gsdl.getDocumentMetadataFromServer(docID, "Number");
            gsdl.getDocumentMetadataFromServer(docID, "Volume");
        }

    }

    public Vector getResultSetMetadata(QueryOutcome queryOutcome) {

        Vector results = queryOutcome.getResults();
        for (Enumeration e = results.elements(); e.hasMoreElements();) {
            Result result = (Result) e.nextElement();
            getResultMetadata(result);
        }
        return results;
    }

    public void initialiseDateMap(Vector results) {
        for (Enumeration e = results.elements(); e.hasMoreElements();) {
            Result result = (Result) e.nextElement();

            String docID = result.getDocID();

            ResultDocument rd = gsdl.getDocument(docID);

            if (rd.metadataExists("Date")) {
                addToDateMap(docID, rd);
            }
        }
    }

    public void addToDateMap(String docID, ResultDocument rd) {
        if (dateMap.containsKey(rd.getDate())) {
            Vector dateVector = (Vector) dateMap.get(rd.getDate());
            if (!dateVector.contains(docID)) {
                dateVector.addElement(docID);
            }
            dateMap.put(rd.getDate(), dateVector);
        } else {
            Vector dateVector = new Vector();
            dateVector.addElement(docID);
            dateMap.put(rd.getDate(), dateVector);
        }
    }

    public void itemStateChanged(ItemEvent e) {
        Object sourceItem = e.getItemSelectable();

        if (sourceItem == stemmingCheck) {
            doStemming = !doStemming;
        } else if (sourceItem == casefoldingCheck) {
            doCasefolding = !doCasefolding;
        }
    }

    public void actionPerformed(ActionEvent e) {
        Object sourceItem = e.getSource();

        if (sourceItem == indexComboBox) {
            indexChoice = indexComboBox.getSelectedIndex();
        } else if (sourceItem == homeButton || sourceItem == waikatoButton) {
            if (homeButton.isSelected()) {
                locationChoice = 1;
            } else {
                locationChoice = 0;
            }
            gsdl = new Greenstone3Connection(locationChoice);
        } else if (sourceItem.getClass().getName().compareTo("javax.swing.JMenuItem") == 0) {
            String menuLabel = ((JMenuItem) sourceItem).getText();
            changeView(menuLabel);
        }
    }

    class MyActionListener implements ActionListener {

        public void actionPerformed(ActionEvent evt) {
            String queryText = queryField.getText();
            doQuery(queryText);
        }
    }
}


