package org.apollo.audio.structure;

/**
 * An immutable class that points to a TrackGraphInfo and gives
 * a absolute initiation time.
 * 
 * @see OverdubbedFrame#getAbsoluteTrackLayoutDeep()
 * 
 * @author Brook Novak
 *
 */
public class AbsoluteTrackNode {
	
	private TrackGraphNode tnode;
	private long absStartTime;
	private String channelID;
	private String parentFrameName;
	
	AbsoluteTrackNode(TrackGraphNode tnode, long absStartTime, String channelID, String parentFrameName) {
		assert(tnode != null);
		assert(channelID != null);
		assert(absStartTime >= 0);
		assert(parentFrameName != null);
		
		this.tnode = tnode;
		this.absStartTime = absStartTime;
		this.channelID = channelID;
		this.parentFrameName = parentFrameName;
	}

	public long getABSStartTime() {
		return absStartTime;
	}

	public TrackGraphNode getTrackNode() {
		return tnode;
	}
	
	public String getChannelID() {
		return channelID;
	}
	
	public String getParentFrameName() {
		return parentFrameName;
	}
}
