package org.apollo.items;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Stroke;

import org.apollo.audio.SampledAudioManager;
import org.apollo.audio.util.FrameLayoutDaemon;
import org.apollo.audio.util.MultiTrackPlaybackController;
import org.apollo.audio.util.SoundDesk;
import org.apollo.audio.util.Timeline;
import org.apollo.audio.util.MultiTrackPlaybackController.MultitrackLoadListener;
import org.apollo.io.IconRepository;
import org.apollo.util.AudioMath;
import org.apollo.widgets.FramePlayer;
import org.expeditee.gui.Browser;
import org.expeditee.gui.DisplayIO;
import org.expeditee.gui.Frame;
import org.expeditee.items.Item;
import org.expeditee.items.ItemParentStateChangedEvent;

/**
 * A transient widget that destroys self once anchored...
 * once anchored it commences playback of a frame if there is something to play.
 * 
 * @author Brook Novak
 *
 */
public class FramePlaybackLauncher extends Item {
	
	private static int WIDTH = 80;
	private static int HEIGHT = 80;
	private static Stroke BORDER_STOKE = new BasicStroke(1.0f);
	
	public FramePlaybackLauncher() {
		this.setID(getParentOrCurrentFrame().getNextItemID());
	}

	@Override
	public Item copy() {
		FramePlaybackLauncher copy = new FramePlaybackLauncher();
		
		DuplicateItem(this, copy);
		
		return copy;
	}

	@Override
	public Item merge(Item merger, int mouseX, int mouseY) {
		return null;
	}

	@Override
	public void paint(Graphics2D g) {
		if (Browser._theBrowser == null) return;
		
		g.setColor(Color.LIGHT_GRAY);
		g.fillRect((int)_x, (int)_y, WIDTH, HEIGHT);
		
		g.setColor(Color.BLACK);
		g.setStroke(BORDER_STOKE);
		g.drawRect((int)_x, (int)_y, WIDTH, HEIGHT);
		
		IconRepository.getIcon("frameplay.png").paintIcon(
				Browser._theBrowser.getContentPane(), g, getX() + 25, getY() + 25);
		
	}

	@Override
	public void setAnnotation(boolean val) {
	}

	
	@Override
	public int getHeight() {
		return HEIGHT;
	}

	@Override
	public int getWidth() {
		return WIDTH;
	}

	@Override
	public void updatePolygon() {
		
		_poly = new Polygon();

		int x = (int)_x;
		int y = (int)_y;
		
		_poly.addPoint(x, y);
		_poly.addPoint(x + WIDTH, y);
		_poly.addPoint(x + WIDTH, y + HEIGHT);
		_poly.addPoint(x, y + HEIGHT);

	}

	@Override
	public void onParentStateChanged(ItemParentStateChangedEvent e) {
		super.onParentStateChanged(e);

		switch (e.getEventType()) {

		case ItemParentStateChangedEvent.EVENT_TYPE_ADDED:
		case ItemParentStateChangedEvent.EVENT_TYPE_ADDED_VIA_OVERLAY:
		case ItemParentStateChangedEvent.EVENT_TYPE_SHOWN:
		case ItemParentStateChangedEvent.EVENT_TYPE_SHOWN_VIA_OVERLAY:
			// todo: Invoke later for concurrent modification possibility?
			selfDestruct();
			break;

		}
	}
	
	private void selfDestruct() {
		
		Frame parent = getParent();
		parent.removeItem(this);
		
		Frame currentFrame = DisplayIO.getCurrentFrame();
		
		if (currentFrame != null) {
			
			Timeline tl = FrameLayoutDaemon.getInstance().getLastComputedTimeline();
			
			if (tl == null || FrameLayoutDaemon.getInstance().getTimelineOwner() == null ||
					FrameLayoutDaemon.getInstance().getTimelineOwner() != currentFrame) {
				tl = FrameLayoutDaemon.inferTimeline(currentFrame);
			}
			
			if (tl != null) {
		
				long initiateMS = tl.getMSTimeAtX((int)_x);
				
				// Clamp
				if (initiateMS < tl.getFirstInitiationTime())
					initiateMS = tl.getFirstInitiationTime();
				
				else if (initiateMS > (tl.getFirstInitiationTime() + tl.getRunningTime())) 
					initiateMS  = tl.getFirstInitiationTime() + tl.getRunningTime();

				int startFrame = AudioMath.millisecondsToFrames(
						initiateMS - tl.getFirstInitiationTime(),  
						SampledAudioManager.getInstance().getDefaultPlaybackFormat());
				
				// To ensure that the frame channels are not in use
				MultiTrackPlaybackController.getInstance().stopPlayback();
				SoundDesk.getInstance().freeChannels(FramePlayer.FRAME_PLAYERMASTER_CHANNEL_ID);

				FramePlayer.playFrame(
						new MultitrackLoadListener() {
		
							public void multiplaybackLoadStatusUpdate(int id, Object state) {
								// Ignore
							}
							
						}, 
						currentFrame.getName(), 
						false, 
						0, 
						startFrame, 
						Integer.MAX_VALUE
						);
			
			}
					
		}
	}
	
	
	
}
