package org.apollo.util;

/**
 * java.lang.Long does not allow:
 * 	Long test = null;
 * 
 * This provides a simple nullable long value.
 * 
 * @author Brook Novak
 *
 */
public class NullableLong implements Comparable<NullableLong> {
	
	private long longValue;
	
	public NullableLong(long value) {
		this.longValue = value;
	}
	
	private NullableLong(Long value) {
		if (value == null) throw new NullPointerException();
		this.longValue = value.longValue();
	}
	
	public static NullableLong fromLong(Long value) {
		if (value == null) return null;
		return new NullableLong(value);
	}

	public int compareTo(NullableLong o) {
		return new Long(longValue).compareTo(new Long(o.longValue));
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null) return false;
		
		if (obj instanceof NullableLong) {
			return longValue == ((NullableLong)obj).longValue;
		}
		
		return false;
	}

	@Override
	public int hashCode() {
		return new Long(longValue).intValue();
	}

	@Override
	public String toString() {
		return Long.toString(longValue);
	}

	public long getLongValue() {
		return longValue;
	}

}
